"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var environment_data_service_1 = require("./environment-data-service");
var logs_process_1 = require("./logs-process");
var state_tracker_proxy_1 = require("./state-tracker-process/state-tracker-proxy");
var request = require("request");
var sl_js_infra_1 = require("sl-js-infra");
var watchdog_1 = require("./watchdog");
var agent_1 = require("./agent");
var logger_1 = require("./logger");
var log_service_proxy_1 = require("./logs-process/log-service-proxy");
var http_coloring_1 = require("./coloring/http-coloring");
var listening_port_1 = require("./coloring/listening-port");
var test_recommendation_handler_1 = require("./test-recommendation-handler");
var BackendProxy = sl_js_infra_1.SlJsInfra.BackendProxy;
var ConfigProcess = sl_js_infra_1.SlJsInfra.ConfigProcess;
var StateTracker = sl_js_infra_1.SlJsInfra.StateTracker;
var Queue = sl_js_infra_1.SlJsInfra.Queue;
var Technology = sl_js_infra_1.SlJsInfra.Entities.Technology;
var agent_instance_data_1 = require("sl-js-infra/tsOutputs/lib/agent-instance-data");
var NodeAgentFactory = /** @class */ (function () {
    function NodeAgentFactory(cfg) {
        if (cfg == null)
            throw new Error("cfg is required");
        NodeAgentFactory.cfg = cfg;
        NodeAgentFactory.getAgentInstanceData(); //Creates a new agent instance (with ID) if required
    }
    NodeAgentFactory.getAgentInstanceData = function () {
        if (!NodeAgentFactory.agentInstanceData) {
            NodeAgentFactory.agentInstanceData = new agent_instance_data_1.AgentInstanceData(sl_js_infra_1.SlJsInfra.Entities.AgentType.TEST_LISTENER, Technology.NODEJS);
        }
        return NodeAgentFactory.agentInstanceData;
    };
    NodeAgentFactory.prototype.getCreateFootprintsHandler = function () {
        if (!NodeAgentFactory.footprintsHandler) {
            if (sl_js_infra_1.SlJsInfra.SlEnvVars.inProductionListenerMode()) {
                NodeAgentFactory.footprintsHandler = new sl_js_infra_1.SlJsInfra.ProductionFootprintsHandler(NodeAgentFactory.cfg, this.getCreateApplicationLogger().child({ component: "ProductionFootprintsHandler" }));
            }
            else {
                NodeAgentFactory.footprintsHandler = new sl_js_infra_1.SlJsInfra.FootprintsProxy(NodeAgentFactory.cfg, this.getCreateApplicationLogger().child({ component: "footprintsHandler" }));
            }
        }
        return NodeAgentFactory.footprintsHandler;
    };
    NodeAgentFactory.prototype.getCreateEventsProcess = function () {
        if (!NodeAgentFactory.eventsProcesss) {
            var sendToServerWatchdog = this.createWatchdog({ autoReset: false, name: "sendToServerWatchdog", interval: NodeAgentFactory.cfg.interval.value, unref: true });
            var keepaliveWatchdog = this.createKeepaliveWatchdog();
            var environmentDataService = this.getCreateEnvironmentDataService();
            var queue = new Queue();
            NodeAgentFactory.eventsProcesss = new sl_js_infra_1.SlJsInfra.EventsProcess(NodeAgentFactory.cfg, NodeAgentFactory.agentInstanceData, sendToServerWatchdog, keepaliveWatchdog, environmentDataService.getEnvDataForSubmission(), this.getCreateBackendProxy(), queue, this.getCreateApplicationLogger().child({ component: "eventsProcess" }));
        }
        ;
        return NodeAgentFactory.eventsProcesss;
    };
    NodeAgentFactory.prototype.getCreateFootprintsProcess = function () {
        var keepaliveWatchdog = this.createKeepaliveWatchdogInfra();
        var logger = this.getCreateApplicationLogger().child({ component: "footprintsProcess" });
        if (!NodeAgentFactory.footprintsProcess) {
            NodeAgentFactory.footprintsProcess = this.shouldCreateFootprintsV6() ?
                this.getCreateFootprintsProcessV6(logger, keepaliveWatchdog) : this.getCreateFootprintsProcessV5(logger, keepaliveWatchdog);
        }
        this.getCreateConfigProcess().on("configuration_changed", function (cfg) {
            NodeAgentFactory.footprintsProcess.updateConfig(cfg);
        });
        return NodeAgentFactory.footprintsProcess;
    };
    NodeAgentFactory.prototype.shouldCreateFootprintsV6 = function () {
        return NodeAgentFactory.cfg.footprintsEnableV6.value && !sl_js_infra_1.SlJsInfra.SlEnvVars.inProductionListenerMode();
    };
    NodeAgentFactory.prototype.getCreateFootprintsProcessV5 = function (logger, keepaliveWatchdog) {
        var sendToServerWatchdog = this.createWatchdogInfra({ autoReset: false, name: "sendToServerWatchdog", interval: NodeAgentFactory.cfg.interval.value, unref: true });
        return new sl_js_infra_1.SlJsInfra.FootprintsProcess(NodeAgentFactory.cfg, sendToServerWatchdog, keepaliveWatchdog, this.getCreateFootprintsHandler(), this.getCreateEnvironmentDataService(), NodeAgentFactory.getAgentInstanceData(), this.getCreateIstanbulToFootprintsCoverter(), this.getCreateStateTracker(), logger);
    };
    NodeAgentFactory.prototype.getCreateFootprintsProcessV6 = function (logger, keepaliveWatchdog) {
        var sendToServerInterval = NodeAgentFactory.cfg.footprintsSendIntervalSecs.value * 1000;
        var sendToServerWatchdog = this.createWatchdogInfra({ autoReset: false, name: "sendToServerWatchdog", interval: sendToServerInterval, unref: true });
        return new sl_js_infra_1.SlJsInfra.FootprintsProcessV6(NodeAgentFactory.cfg, sendToServerWatchdog, keepaliveWatchdog, logger, this.getCreateHitsCollector(), this.getCreateHitsConverter(), this.getCreateFootprintsBuffer(), this.getCreateBackendProxy(), this.getCreateStateTracker());
    };
    NodeAgentFactory.prototype.createWatchdog = function (opts) {
        return new watchdog_1.Watchdog(opts, { setTimeout: setTimeout, clearTimeout: clearTimeout });
    };
    NodeAgentFactory.prototype.createWatchdogInfra = function (opts) {
        return new sl_js_infra_1.SlJsInfra.Watchdog(opts, { setTimeout: setTimeout, clearTimeout: clearTimeout });
    };
    NodeAgentFactory.prototype.createKeepaliveWatchdog = function () {
        return this.createWatchdog({ autoReset: true, name: "keepalive", interval: 1000, unref: false });
    };
    NodeAgentFactory.prototype.createKeepaliveWatchdogInfra = function () {
        var opt = { autoReset: true, name: "keepalive", interval: 1000, unref: false };
        return new sl_js_infra_1.SlJsInfra.Watchdog(opt, { setTimeout: setTimeout, clearTimeout: clearTimeout });
    };
    NodeAgentFactory.prototype.getCreateConfigProcess = function () {
        if (!NodeAgentFactory.configProcess) {
            var sendToServerWatchdog = this.createWatchdogInfra({ autoReset: true, name: "getConfigFromServerWatchdog", interval: 10 * 60 * 1000, unref: true });
            NodeAgentFactory.configProcess = new ConfigProcess(NodeAgentFactory.cfg, sendToServerWatchdog, this.getCreateBackendProxy(), this.getCreateApplicationLogger().child({ component: "configProcess" }));
        }
        return NodeAgentFactory.configProcess;
    };
    NodeAgentFactory.prototype.getCreateEnvironmentDataService = function () {
        if (!NodeAgentFactory.environmentDataService) {
            NodeAgentFactory.environmentDataService = new environment_data_service_1.EnvironmentDataService(NodeAgentFactory.cfg);
        }
        return NodeAgentFactory.environmentDataService;
    };
    NodeAgentFactory.prototype.getCreateNodeJsAgent = function () {
        if (!NodeAgentFactory.nodeJsAgent) {
            var interval = NodeAgentFactory.cfg.footprintsEnableV6.value ?
                NodeAgentFactory.cfg.footprintsCollectIntervalSecs.value * 1000 : NodeAgentFactory.cfg.interval.value;
            NodeAgentFactory.nodeJsAgent = new agent_1.NodeJsAgent(NodeAgentFactory.cfg, this.getCreateFootprintsProcess(), this.getCreateEventsProcess(), this.getCreateLogsProcess(), this.getCreateStateTracker(), this.getCreateConfigProcess(), this.createWatchdog({ interval: interval, unref: true, name: "flushCurrentFootprints", autoReset: true }), this.getCreateColoringProviders(), this.getCreateTestRecommendationHandler(), NodeAgentFactory.getAgentInstanceData(), this.getCreateApplicationLogger().child({ component: "main" }));
        }
        return NodeAgentFactory.nodeJsAgent;
    };
    NodeAgentFactory.prototype.getCreateLogServiceProxy = function () {
        if (!NodeAgentFactory.logServiceProxy) {
            NodeAgentFactory.logServiceProxy = new log_service_proxy_1.LogServiceProxy(NodeAgentFactory.cfg, request, this.getCreateApplicationLogger().child({ component: "logsServiceProxy" }));
        }
        return NodeAgentFactory.logServiceProxy;
    };
    NodeAgentFactory.prototype.getCreateLogsProcess = function () {
        if (!NodeAgentFactory.logsProcess) {
            var sendToServerWatchdog = this.createWatchdog({ autoReset: false, name: "sendToServerWatchdog", interval: NodeAgentFactory.cfg.interval.value, unref: true });
            var keepaliveWatchdog = this.createKeepaliveWatchdog();
            NodeAgentFactory.logsProcess = new logs_process_1.LogsProcess(NodeAgentFactory.cfg, this.getCreateLoggerFactory().getCreateBunyanBufferingLogger(), this.getCreateLogServiceProxy(), this.getCreateEnvironmentDataService(), NodeAgentFactory.getAgentInstanceData(), sendToServerWatchdog, keepaliveWatchdog, this.getCreateApplicationLogger().child({ component: "logsProcess" }));
        }
        return NodeAgentFactory.logsProcess;
    };
    NodeAgentFactory.prototype.getCreateStateTracker = function () {
        if (!NodeAgentFactory.stateTracker) {
            var interval = NodeAgentFactory.cfg.footprintsEnableV6.value ?
                NodeAgentFactory.cfg.executionQueryIntervalSecs.value * 1000 : NodeAgentFactory.cfg.interval.value;
            var executionQueryWatchdog = this.createWatchdogInfra({
                autoReset: true,
                interval: interval,
                name: "testStatus",
                unref: true
            });
            NodeAgentFactory.stateTracker = new StateTracker(NodeAgentFactory.cfg, this.getCreateConfigProcess(), executionQueryWatchdog, this.getCreateBackendProxy(), this.getCreateApplicationLogger().child({ component: "stateTrackerProcess" }));
        }
        return NodeAgentFactory.stateTracker;
    };
    NodeAgentFactory.prototype.getCreateStateTrackerProxy = function () {
        if (!NodeAgentFactory.stateTrackerProxy) {
            NodeAgentFactory.stateTrackerProxy = new state_tracker_proxy_1.StateTrackerProxy(NodeAgentFactory.cfg, request, this.getCreateApplicationLogger().child({ component: "stateTrackerProxy" }));
        }
        return NodeAgentFactory.stateTrackerProxy;
    };
    NodeAgentFactory.prototype.getCreateIstanbulToFootprintsCoverter = function () {
        if (!NodeAgentFactory.istanbulToFootprintsConverter) {
            var logger = this.getCreateApplicationLogger().child({ component: "IstanbulToFootprintsV3" });
            NodeAgentFactory.istanbulToFootprintsConverter = new sl_js_infra_1.SlJsInfra.IstanbulToFootprints(NodeAgentFactory.cfg, logger);
        }
        return NodeAgentFactory.istanbulToFootprintsConverter;
    };
    NodeAgentFactory.prototype.getCreateLoggerFactory = function () {
        if (NodeAgentFactory.loggerFactory == null) {
            NodeAgentFactory.loggerFactory = new logger_1.LoggerFactory(NodeAgentFactory.cfg);
        }
        return NodeAgentFactory.loggerFactory;
    };
    NodeAgentFactory.prototype.getCreateApplicationLogger = function () {
        if (NodeAgentFactory.applicationLogger == null) {
            NodeAgentFactory.applicationLogger = this.getCreateLoggerFactory().getCreateApplicationLogger();
        }
        return NodeAgentFactory.applicationLogger;
    };
    NodeAgentFactory.prototype.getCreateColoringProviders = function () {
        if (!NodeAgentFactory.coloringProviders) {
            NodeAgentFactory.coloringProviders = new Array();
            NodeAgentFactory.coloringProviders.push(new http_coloring_1.HttpColoring());
            NodeAgentFactory.coloringProviders.push(new listening_port_1.ListeningPort());
        }
        return NodeAgentFactory.coloringProviders;
    };
    NodeAgentFactory.prototype.getCreateTestRecommendationHandler = function () {
        if (!NodeAgentFactory.testRecommendationHandler) {
            var buildSessionId = NodeAgentFactory.cfg.buildSessionId.value;
            var testStage = NodeAgentFactory.cfg.testStage.value;
            var enforceFullRun = NodeAgentFactory.cfg.enforceFullRun.value;
            NodeAgentFactory.testRecommendationHandler = new test_recommendation_handler_1.TestRecommendationHandler(buildSessionId, testStage, enforceFullRun, this.getCreateBackendProxy(), this.getCreateEventsProcess(), this.getCreateApplicationLogger());
        }
        return NodeAgentFactory.testRecommendationHandler;
    };
    NodeAgentFactory.prototype.getCreateBackendProxy = function () {
        if (!NodeAgentFactory.backendProxy) {
            var config = {
                token: NodeAgentFactory.cfg.token.value,
                proxy: NodeAgentFactory.cfg.proxy.value,
                server: NodeAgentFactory.cfg.server.value,
                compressRequests: NodeAgentFactory.cfg.gzip.value
            };
            NodeAgentFactory.backendProxy = new BackendProxy(NodeAgentFactory.agentInstanceData, config, this.getCreateApplicationLogger());
        }
        return NodeAgentFactory.backendProxy;
    };
    NodeAgentFactory.prototype.getCreateFootprintsBuffer = function () {
        if (!NodeAgentFactory.footprintsBuffer) {
            NodeAgentFactory.footprintsBuffer = new sl_js_infra_1.SlJsInfra.FootprintsBuffer(NodeAgentFactory.agentInstanceData, NodeAgentFactory.cfg);
        }
        return NodeAgentFactory.footprintsBuffer;
    };
    NodeAgentFactory.prototype.getCreateRelativePathResolver = function () {
        if (!NodeAgentFactory.relativePathResolver) {
            NodeAgentFactory.relativePathResolver = new sl_js_infra_1.SlJsInfra.RelativePathResolver(NodeAgentFactory.cfg.projectRoot.value);
        }
        return NodeAgentFactory.relativePathResolver;
    };
    NodeAgentFactory.prototype.getCreateSourceMapData = function () {
        if (!NodeAgentFactory.sourceMapData) {
            NodeAgentFactory.sourceMapData = new sl_js_infra_1.SlJsInfra.SourceMapData(this.getCreateApplicationLogger().child({ component: "SourceMapData" }));
        }
        return NodeAgentFactory.sourceMapData;
    };
    NodeAgentFactory.prototype.getCreateHitsCollector = function () {
        if (!NodeAgentFactory.hitsCollector) {
            NodeAgentFactory.hitsCollector = new sl_js_infra_1.SlJsInfra.HitsCollector(this.getCreateApplicationLogger().child({ component: "HitsCollector" }));
        }
        return NodeAgentFactory.hitsCollector;
    };
    NodeAgentFactory.prototype.getCreateHitsConverter = function () {
        if (!NodeAgentFactory.hitsConverter) {
            NodeAgentFactory.hitsConverter = new sl_js_infra_1.SlJsInfra.HitsConverter(this.getCreateRelativePathResolver(), this.getCreateSourceMapData(), NodeAgentFactory.cfg.projectRoot.value, this.getCreateApplicationLogger().child({ component: "HitsConverter" }));
        }
        return NodeAgentFactory.hitsConverter;
    };
    /*
        For tests only. reset all static members to isolate tests
     */
    NodeAgentFactory.resetAll = function () {
        this.cfg = null;
        this.footprintsHandler = null;
        this.eventsProcesss = null;
        this.footprintsProcess = null;
        this.configProcess = null;
        this.environmentDataService = null;
        this.nodeJsAgent = null;
        this.logServiceProxy = null;
        this.logsProcess = null;
        this.stateTracker = null;
        this.stateTrackerProxy = null;
        this.istanbulToFootprintsConverter = null;
        this.loggerFactory = null;
        this.applicationLogger = null;
        this.coloringProviders = null;
    };
    return NodeAgentFactory;
}());
exports.NodeAgentFactory = NodeAgentFactory;
//# sourceMappingURL=factory.js.map